@@ -39,14 +39,35 @@ mod show;
 
 #[derive(Debug, StructOpt)]
 struct Aware {
-    #[structopt(long, short)]
+    #[structopt(
+        help = "Explore resources from this region / these regions",
+        long,
+        short,
+        global = true
+    )]
     region: Vec<String>,
-    #[structopt(long, short)]
-    vpc: Vec<String>,
+    #[structopt(subcommand)]
+    service: AwsService,
+}
+
+#[derive(Debug, StructOpt)]
+pub(crate) enum AwsService {
+    #[structopt(name = "ec2", about = "Explore EC2 resources")]
+    Ec2 {
+        #[structopt(long, short)]
+        vpc: Vec<String>,
+    },
+    #[structopt(name = "cf", about = "Explore CloudFormation resources")]
+    CloudFormation {
+        #[structopt(help = "Filter by given stack name", long)]
+        stack: Vec<String>,
+        #[structopt(help = "Filter by given stack status", long)]
+        status: Vec<aws::cf::model::StackStatus>,
+    },
 }
 
 #[tokio::main]
-async fn main() -> Result<(), ec2::Error> {
+async fn main() -> anyhow::Result<()> {
     let aware = Aware::from_args();
 
     let regions = if aware.region.is_empty() {
@@ -54,7 +75,11 @@ async fn main() -> Result<(), ec2::Error> {
     } else {
         aware.region
     };
-    collect(regions, aware.vpc).await
+
+    match aware.service {
+        AwsService::Ec2 { vpc } => collect_ec2(regions, vpc).await,
+        AwsService::CloudFormation { stack, status } => collect_cf(regions, stack, status).await,
+    }
 }
 
 async fn get_all_regions() -> Result<Vec<String>, ec2::Error> {
@@ -70,7 +95,7 @@ async fn get_all_regions() -> Result<Vec<String>, ec2::Error> {
     Ok(regions)
 }
 
-async fn collect(regions: Vec<String>, vpc: Vec<String>) -> Result<(), ec2::Error> {
+async fn collect_ec2(regions: Vec<String>, vpc: Vec<String>) -> anyhow::Result<()> {
     let regioned_clients = regions
         .into_iter()
         .map(ec2::Region::new)
@@ -87,7 +112,7 @@ async fn collect(regions: Vec<String>, vpc: Vec<String>) -> Result<(), ec2::Erro
             ),
         );
         progress.set_prefix(region.clone());
-        let mut aws = aws::AwsResources::new(client);
+        let mut aws = aws::Ec2Resources::new(client);
         progress.set_message("Collecting VPCs");
         aws.collect_vpcs(&vpc).await?;
         progress.inc(1);
@@ -108,7 +133,46 @@ async fn collect(regions: Vec<String>, vpc: Vec<String>) -> Result<(), ec2::Erro
     Ok(())
 }
 
-fn vpc_tree(aws: &aws::AwsResources, vpc: &ec2::model::Vpc) -> ptree::item::StringItem {
+async fn collect_cf(
+    regions: Vec<String>,
+    stack: Vec<String>,
+    status: Vec<aws::cf::model::StackStatus>,
+) -> anyhow::Result<()> {
+    let regioned_clients = regions
+        .into_iter()
+        .map(ec2::Region::new)
+        .map(RegionProviderChain::first_try);
+    let statuses = aws::cf::adjust_stack_statuses(status);
+
+    for region in regioned_clients {
+        let shared_config = aws_config::from_env().region(region).load().await;
+        let region = format!("AWS Region {:?}", shared_config.region().id_and_name());
+        // let client = cf::Client::new(&shared_config);
+
+        let progress = indicatif::ProgressBar::new(1)
+            .with_style(indicatif::ProgressStyle::default_bar().template(
+            "[{pos:>3}/{len:>3} {prefix}] {msg:24!} {wide_bar} [{elapsed}/{duration} ETA {eta}]",
+        ));
+        progress.set_prefix(region.clone());
+        let mut cf = aws::CfResources::new(&shared_config);
+        progress.set_message("Collecting stacks");
+        cf.collect_stacks(&stack, &statuses).await?;
+        progress.inc(1);
+
+        cf.collect_stack_resources(&progress).await?;
+
+        progress.finish();
+
+        cf.trees().for_each(|tree| {
+            println!();
+            ptree::print_tree(&tree).expect("Failed to print tree");
+        });
+    }
+
+    Ok(())
+}
+
+fn vpc_tree(aws: &aws::Ec2Resources, vpc: &ec2::model::Vpc) -> ptree::item::StringItem {
     let mut tree = ptree::TreeBuilder::new(vpc.id_and_name());
     let tree = &mut tree;
     let vpc_id = vpc.id();
